package com.menagerie.ophelia

import android.os.Bundle
import androidx.activity.ComponentActivity
import androidx.activity.compose.setContent
import androidx.compose.foundation.Image
import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.Box
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.padding
import androidx.compose.material3.Surface
import androidx.compose.material3.Text
import androidx.compose.runtime.Composable
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.layout.ContentScale
import androidx.compose.ui.res.painterResource
import androidx.compose.ui.res.stringResource
import androidx.compose.ui.text.style.TextAlign
import androidx.compose.ui.tooling.preview.Preview
import androidx.compose.ui.unit.dp
import androidx.compose.ui.unit.sp
import com.menagerie.ophelia.ui.theme.OpheliaTheme

data class Question<T>(
    val questionText: String,
    val answer: T,
    val difficulty: Difficulty
)

enum class Difficulty {
    EASY, MEDIUM, HARD
}


interface ProgressPrintable {
    val progressText: String
    fun printProgressBar(): String
}

class Quiz : ProgressPrintable {

    override val progressText: String
        get() = "$answered of $total answered."

    override fun printProgressBar(): String {
        var progress = ""
        repeat(Quiz.answered) { progress += "▓" }
        repeat(Quiz.total - Quiz.answered) {progress += "▒"}
        return progress
    }

    fun printQuiz(): String {
        var quizText: String = ""
        question1.let {
            quizText += it.questionText + "\n"
            quizText += it.answer +"\n"
            quizText += it.difficulty.toString() + "\n"
        }
        quizText += "\n"
        question2.let {
            quizText += it.questionText + "\n"
            quizText += it.answer.toString() + "\n"
            quizText += it.difficulty.toString() + "\n"
        }
        quizText += "\n"
        question3.let {
            quizText += it.questionText + "\n"
            quizText += it.answer.toString() +"\n"
            quizText += it.difficulty.toString() + "\n"
        }
        return quizText
    }

    private val question1 = Question<String> (
            "Quoth the raven ___",
            "nevermore",
            Difficulty.MEDIUM
        )
    private val question2 = Question<Boolean> (
        "The sky is green. True or false",
        false,
        Difficulty.EASY
    )
    private val question3 = Question<Int> (
        "How many days are there between full moons?",
            28,
            Difficulty.HARD
        )

    //Declaring Progress a "companion" is what lets us directly access total and
    //answered from Quiz
    companion object Progress {
        var total: Int = 10
        var answered: Int = 5
    }
}

class MainActivity : ComponentActivity() {
    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContent {
            OpheliaTheme {
                // A surface container using the 'background' color from the theme
                QuestionGame()
            }
        }
    }
}

@Composable
fun QuestionGame(modifier: Modifier = Modifier) {

    //just start doing stuff right away with .apply{}
    val quiz = Quiz().apply {
        Quiz.total = 15
    }

    //You can even apply it to a class directly
    //and since Progress is an "object" or singleton,
    //it still updates what we display in the preview!
    Quiz().apply {
        Quiz.answered = 7
    }

    Column(
        modifier = Modifier.fillMaxSize(),
    ) {
        var text = quiz.printProgressBar()
        Text(text = text)
        Text(
            text = quiz.progressText //Because Progress is a companion, this is accessible, too!
        )
        Text(
            text = quiz.printQuiz()
        )
    }
}

@Composable
fun GreetingImage(message: String, from: String, modifier: Modifier = Modifier) {
    val image = painterResource(R.drawable.androidparty)
    Box {
        Image(
            painter = image,
            contentDescription = null,
            contentScale = ContentScale.Crop,
            alpha = 0.5F,
        )
        GreetingText(
            message = message,
            from = from,
            modifier = Modifier
                .fillMaxSize()
                .padding(8.dp)
        )
    }
}

@Composable
fun GreetingText(message: String, from: String, modifier: Modifier = Modifier) {
    // Create a column so that texts don't overlap
    Column(
        verticalArrangement = Arrangement.Center,
        modifier = modifier
    ) {
        Text(
            text = message,
            color = Color.Black,
            fontSize = 70.sp,
            lineHeight = 116.sp,
            textAlign = TextAlign.Center
        )
        Text(
            text = from,
            color = Color.Black,
            fontSize = 36.sp,
            modifier = Modifier
                .padding(16.dp)
                .align(alignment = Alignment.CenterHorizontally)
        )
    }
}

//@Preview(showBackground = true)
@Composable
fun BirthdayCardPreview() {
    OpheliaTheme {
        GreetingImage(
            message = stringResource(R.string.main_welcome_message),
            from = stringResource(R.string.tagline)
        )
    }
}

@Preview(showBackground = true)
@Composable
fun QuestionGamePreview() {
    OpheliaTheme {
        QuestionGame()
    }
}